"""module containing commands for manipulating filters in scenes."""

from typing import Annotated, Optional

import obsws_python as obsws
import typer
from rich.console import Console
from rich.table import Table

from . import util
from .alias import AliasGroup

app = typer.Typer(cls=AliasGroup)
out_console = Console()
err_console = Console(stderr=True, style='bold red')


@app.callback()
def main():
    """Control filters in OBS scenes."""


@app.command('list | ls')
def list_(
    ctx: typer.Context,
    source_name: Annotated[
        Optional[str],
        typer.Argument(
            show_default='The current scene',
            help='The source to list filters for',
        ),
    ] = None,
):
    """List filters for a source."""
    if not source_name:
        source_name = ctx.obj.get_current_program_scene().scene_name

    try:
        resp = ctx.obj.get_source_filter_list(source_name)
    except obsws.error.OBSSDKRequestError as e:
        if e.code == 600:
            err_console.print(
                f'No source was found by the name of [yellow]{source_name}[/yellow].'
            )
            raise typer.Exit(1)
        else:
            raise

    if not resp.filters:
        out_console.print(f'No filters found for source [yellow]{source_name}[/yellow]')
        raise typer.Exit()

    table = Table(title=f'Filters for Source: {source_name}', padding=(0, 2))

    for column in ('Filter Name', 'Kind', 'Enabled', 'Settings'):
        table.add_column(
            column,
            justify='left' if column in ('Filter Name', 'Kind') else 'center',
            style='cyan',
        )

    for filter in resp.filters:
        resp = ctx.obj.get_source_filter_default_settings(filter['filterKind'])
        settings = resp.default_filter_settings | filter['filterSettings']

        table.add_row(
            filter['filterName'],
            util.snakecase_to_titlecase(filter['filterKind']),
            ':white_heavy_check_mark:' if filter['filterEnabled'] else ':x:',
            '\n'.join(
                [
                    f'{util.snakecase_to_titlecase(k):<20} {v:>10}'
                    for k, v in settings.items()
                ]
            ),
        )

    out_console.print(table)


def _get_filter_enabled(ctx: typer.Context, source_name: str, filter_name: str):
    """Get the status of a filter for a source."""
    resp = ctx.obj.get_source_filter(source_name, filter_name)
    return resp.filter_enabled


@app.command('enable | on')
def enable(
    ctx: typer.Context,
    source_name: Annotated[
        str,
        typer.Argument(
            ..., show_default=False, help='The source to enable the filter for'
        ),
    ],
    filter_name: Annotated[
        str,
        typer.Argument(
            ..., show_default=False, help='The name of the filter to enable'
        ),
    ],
):
    """Enable a filter for a source."""
    if _get_filter_enabled(ctx, source_name, filter_name):
        err_console.print(
            f'Filter [yellow]{filter_name}[/yellow] is already enabled for source [yellow]{source_name}[/yellow]'
        )
        raise typer.Exit(1)

    ctx.obj.set_source_filter_enabled(source_name, filter_name, enabled=True)
    out_console.print(
        f'Enabled filter [green]{filter_name}[/green] for source [green]{source_name}[/green]'
    )


@app.command('disable | off')
def disable(
    ctx: typer.Context,
    source_name: Annotated[
        str,
        typer.Argument(
            ..., show_default=False, help='The source to disable the filter for'
        ),
    ],
    filter_name: Annotated[
        str,
        typer.Argument(
            ..., show_default=False, help='The name of the filter to disable'
        ),
    ],
):
    """Disable a filter for a source."""
    if not _get_filter_enabled(ctx, source_name, filter_name):
        err_console.print(
            f'Filter [yellow]{filter_name}[/yellow] is already disabled for source [yellow]{source_name}[/yellow]'
        )
        raise typer.Exit(1)

    ctx.obj.set_source_filter_enabled(source_name, filter_name, enabled=False)
    out_console.print(
        f'Disabled filter [green]{filter_name}[/green] for source [green]{source_name}[/green]'
    )


@app.command('toggle | tg')
def toggle(
    ctx: typer.Context,
    source_name: Annotated[
        str,
        typer.Argument(
            ..., show_default=False, help='The source to toggle the filter for'
        ),
    ],
    filter_name: Annotated[
        str,
        typer.Argument(
            ..., show_default=False, help='The name of the filter to toggle'
        ),
    ],
):
    """Toggle a filter for a source."""
    is_enabled = _get_filter_enabled(ctx, source_name, filter_name)
    new_state = not is_enabled

    ctx.obj.set_source_filter_enabled(source_name, filter_name, enabled=new_state)
    if new_state:
        out_console.print(
            f'Enabled filter [green]{filter_name}[/green] for source [green]{source_name}[/green]'
        )
    else:
        out_console.print(
            f'Disabled filter [green]{filter_name}[/green] for source [green]{source_name}[/green]'
        )


@app.command('status | ss')
def status(
    ctx: typer.Context,
    source_name: Annotated[
        str,
        typer.Argument(
            ..., show_default=False, help='The source to get the filter status for'
        ),
    ],
    filter_name: Annotated[
        str,
        typer.Argument(
            ..., show_default=False, help='The name of the filter to get the status for'
        ),
    ],
):
    """Get the status of a filter for a source."""
    is_enabled = _get_filter_enabled(ctx, source_name, filter_name)
    if is_enabled:
        out_console.print(
            f'Filter [green]{filter_name}[/green] is enabled for source [green]{source_name}[/green]'
        )
    else:
        out_console.print(
            f'Filter [green]{filter_name}[/green] is disabled for source [green]{source_name}[/green]'
        )
